import logging

from django.core.urlresolvers import reverse_lazy
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.db import transaction
from django.http import HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _

from django.views.generic import CreateView
from django.views.generic import DeleteView
from django.views.generic import DetailView
from django.views.generic import RedirectView
from django.views.generic import TemplateView

from change_email.conf import settings
from change_email.forms import EmailChangeForm
from change_email.models import EmailChange
from change_email.signals import email_change_confirmed
from change_email.signals import email_change_created
from change_email.signals import email_change_deleted


logger = logging.getLogger(__name__)


class EmailChangeConfirmView(TemplateView):
    """
A view to confirm an email address change request.
"""
    template_name = "change_email/emailchange_confirm.html"
    """The template used to render this view."""

    object = None
    """An instance of :model:`EmailChange`, if found."""

    def __init__(self, *args, **kwargs):
        super(EmailChangeConfirmView, self).__init__(*args, **kwargs)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """
If an :model:`EmailChange` object that
has been created by the user is not found, the user will be
redirected to :view:`EmailChangeCreateView`.
"""
        try:
            object = EmailChange.objects.filter(user=request.user).get()
        except EmailChange.DoesNotExist:
            msg = _("No email address change request was found. Either an old "
                    "one has expired or a new one has not been requested.")
            messages.add_message(request,
                                 messages.ERROR,
                                 msg,
                                 fail_silently=True)
            logger.error('No email address change request found.')
            #return HttpResponseRedirect(reverse_lazy('change_email_create'))
            object = None
        self.object = object
        return super(EmailChangeConfirmView, self).dispatch(request,
                                                            *args,
                                                            **kwargs)

    def get_context_data(self, **kwargs):
        """
Inserts following variables into the context:

``object``
    An instance of :model:`EmailChange`.

``confirmed``
    A boolean determining if the change request has been confirmed
    succesfully.
"""
        kwargs['object'] = self.object
        kwargs['confirmed'] = False
        if self.object and self.object.check_signature(self.kwargs['signature']):
            kwargs['confirmed'] = True
            self.save()
        if kwargs['confirmed']:
            msg = _("The email address change was confirmed. Your new email"
                    " address will be used as primary address.")
            messages.add_message(self.request,
                                 messages.INFO,
                                 msg,
                                 fail_silently=True)
        else:
            msg = _("The email address change has not been confirmed."
                    " Please request a new one.")
            messages.add_message(self.request,
                                 messages.ERROR,
                                 msg,
                                 fail_silently=True)
            logger.error('Email address change request was not confirmed.')
        return super(EmailChangeConfirmView, self).get_context_data(**kwargs)

    def save(self):
        """
Saves the new email address to :class:`django.contrib.auth.models.User` and
send a :signal:`email_change_confirmed` signal.
"""
        setattr(self.request.user, settings.EMAIL_CHANGE_FIELD,
                self.object.new_email)
        #self.request.user.email = self.object.new_email
        self.request.user.save()
        self.object.delete()
        email_change_confirmed.send(sender=self, request=self.request)


class EmailChangeCreateView(CreateView):
    """
A view to create an :model:`EmailChange` object.
"""
    model = EmailChange

    form_class = EmailChangeForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """
If an :model:`EmailChange` object that
has been created by the user is found, the user will be
redirected to :view:`EmailChangeDetailView`.
"""
        if EmailChange.objects.filter(user=request.user).count():
            msg = _("An email address change request was found. It must"
                    " be deleted before a new one can be requested.")
            messages.add_message(request,
                                 messages.ERROR,
                                 msg,
                                 fail_silently=True)
            logger.error('Pending email address change request found.')
            object = EmailChange.objects.filter(user=request.user).get()
            return HttpResponseRedirect(reverse_lazy('change_email_detail',
                                                     args=[object.pk]))
        return super(EmailChangeCreateView, self).dispatch(request,
                                                           *args,
                                                           **kwargs)

    def form_valid(self, form):
        """
Saves the email address change request, sends an email to confirm the
request, adds a success message for the user and redirects to
:view:`EmailChangeDetailView`.
"""
        form.instance.user = self.request.user
        instance = super(EmailChangeCreateView, self).form_valid(form)
        msg = _("The email address change request was processed.")
        messages.add_message(self.request,
                             messages.INFO,
                             msg,
                             fail_silently=True)
        email_change_created.send(sender=self, request=self.request)
        form.instance.send_confirmation_mail(self.request)
        return instance
    form_valid = transaction.commit_on_success(form_valid)


class EmailChangeDeleteView(DeleteView):
    """
A view to delete an :model:`EmailChange` object.
"""
    model = EmailChange

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """
If an :model:`EmailChange` object that
has been created by the user is not found, the user will be
redirected to :view:`EmailChangeCreateView`.
"""
        if not EmailChange.objects.filter(user=request.user).count():
            msg = _("No email address change request was found. Either an "
                    "old one has expired or a new one has not been requested.")
            messages.add_message(request,
                                 messages.ERROR,
                                 msg,
                                 fail_silently=True)
            logger.error('No email address change request found.')
            return HttpResponseRedirect(reverse_lazy('change_email_create'))
        return super(EmailChangeDeleteView, self).dispatch(request,
                                                           *args,
                                                           **kwargs)

    def get_success_url(self, **kwargs):
        """
Returns the URL to redirect to after an email address change request has
been deleted by an user. The URL to redirect to can be customized by setting
:py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_DELETE_SUCCESS_REDIRECT_URL`.
"""
        msg = _("The email address change request was deleted.")
        messages.add_message(self.request,
                             messages.INFO,
                             msg,
                             fail_silently=True)
        email_change_deleted.send(sender=self, request=self.request)
        return reverse_lazy('change_email_create')


class EmailChangeDetailView(DetailView):
    """
A view to display an :model:`EmailChange` object.
"""
    model = EmailChange

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """
If an :model:`EmailChange` object that has been created by the user is not
found, the user will be redirected to :view:`EmailChangeCreateView`.
"""
        if not EmailChange.objects.filter(user=request.user).count():
            msg = _("No email address change request was found. Either an "
                    "old one has expired or a new one has not been requested.")
            messages.add_message(request,
                                 messages.ERROR,
                                 msg,
                                 fail_silently=True)
            logger.error('No email address change request found.')
            return HttpResponseRedirect(reverse_lazy('change_email_create'))
        return super(EmailChangeDetailView, self).dispatch(request,
                                                           *args,
                                                           **kwargs)


class EmailChangeIndexView(RedirectView):
    """
A view to redirect users to other views.
"""
    permanent = False
    """
Determines that this view will always issue a HTTP 307 (Temporary
Redirect) status code.
"""

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(EmailChangeIndexView, self).dispatch(request,
                                                          *args,
                                                          **kwargs)

    def get_redirect_url(self, **kwargs):
        """
If an :model:`EmailChange` object is found that
has been created by the user, the user will be redirected to
:view:`EmailChangeDetailView`. If such an
object is not found the user will be redirected
to :view:`EmailChangeCreateView`.
"""
        user = self.request.user
        try:
            object = EmailChange.objects.filter(user=user).get()
            return reverse_lazy('change_email_detail', args=[object.pk])
        except EmailChange.DoesNotExist:
            return reverse_lazy('change_email_create')



from django import forms
from django.utils.translation import ugettext_lazy as _

from change_email.models import EmailChange
from change_email.validators import validate_email_not_used


class EmailChangeForm(forms.ModelForm):
    """
A form to allow users to change the email address they have
registered with.

Just consists of an ``forms.EmailField`` with a
:validator:`validate_email_not_used` validator to check if a
given email address is not already used.
"""
    new_email = forms.EmailField(help_text=_('Please enter your'
                                             ' new email address.'),
                                 label=_('new email address'),
                                 validators=[validate_email_not_used])

    class Meta:
        model = EmailChange
        exclude = ('user', 'site')



from django.conf.urls.defaults import *

from change_email.views import EmailChangeConfirmView
from change_email.views import EmailChangeCreateView
from change_email.views import EmailChangeDeleteView
from change_email.views import EmailChangeDetailView
from change_email.views import EmailChangeIndexView

urlpatterns = patterns('',
                       url(r'^change/$',
                           EmailChangeIndexView.as_view(),
                           name='change_email_index'),
                       url(r'^change/confirm/(?P<signature>[0-9A-Za-z-_=]{1,40})/$',
                           EmailChangeConfirmView.as_view(),
                           name='change_email_confirm'),
                       url(r'^change/create/$',
                           EmailChangeCreateView.as_view(),
                           name='change_email_create'),
                       url(r'^change/delete/(?P<pk>\d+)/$',
                           EmailChangeDeleteView.as_view(),
                           name='change_email_delete'),
                       url(r'^change/(?P<pk>\d+)/$',
                           EmailChangeDetailView.as_view(),
                           name='change_email_detail'),
                       )



import datetime

from django.db import models

from change_email.conf import settings


class ExpiredEmailChangeManager(models.Manager):
    def get_query_set(self):
        """
Returns all instances that are older
than :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_TIMEOUT`.
"""
        delta = datetime.timedelta(seconds=settings.EMAIL_CHANGE_TIMEOUT)
        expiration_date = datetime.datetime.now() - delta
        queryset = super(ExpiredEmailChangeManager, self).get_query_set()
        date = expiration_date.strftime("%Y-%m-%d %H:%M:%S")
        return queryset.filter(date__lte=date)


class PendingEmailChangeManager(models.Manager):
    def get_query_set(self):
        """
Returns all instances that are newer
than :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_TIMEOUT`.
"""
        delta = datetime.timedelta(seconds=settings.EMAIL_CHANGE_TIMEOUT)
        expiration_date = datetime.datetime.now() - delta
        queryset = super(PendingEmailChangeManager, self).get_query_set()
        date = expiration_date.strftime("%Y-%m-%d %H:%M:%S")
        return queryset.filter(date__gt=date)



import time
import datetime

from django.contrib.auth.models import User
from django.core import management

from change_email.conf import settings
from change_email.models import EmailChange
from change_email.tests.lib import BaseTest


class EmailChangeModelTestCase(BaseTest):

    fixtures = ['django_change_email_test_models_fixtures.json']

    def setUp(self):
        output = super(EmailChangeModelTestCase, self).setUp()
        self.alice = User.objects.get(username='alice')
        self.bob = User.objects.get(username='bob')
        self.timeout_days = settings.EMAIL_CHANGE_TIMEOUT
        return output

    def test_email_address_change_creation(self):
        """
        Creating a new email address change request populates the correct data.

        """
        request = EmailChange.objects.create(new_email='bob2@example.com',
                                             user=self.bob)
        self.assertEqual(request.user.id, self.bob.id)
        self.assertEqual(request.new_email, 'bob2@example.com')
        self.failIf(request.has_expired())
        request.delete()

    def test_email_address_change_has_expired(self):
        """
        Testing the model's token methods.

        """
        request1 = EmailChange.objects.create(new_email='bob2@example.com',
                                              user=self.bob)
        time.sleep(2)
        self.failUnless(request1.has_expired(seconds=1))
        self.failIf(request1.has_expired(seconds=1000))


    def test_email_address_change_management_command(self):
        """
        Testing the management command.

        """
        request1 = EmailChange.objects.create(new_email='bob2@example.com',
                                              user=self.bob)
        self.assertEqual(request1.user.id, self.bob.id)
        self.assertEqual(request1.new_email, 'bob2@example.com')
        self.failIf(request1.has_expired())
        request2 = EmailChange.objects.create(new_email='alice2@example.com',
                                              user=self.alice)
        self.assertEqual(request2.user.id, self.alice.id)
        self.assertEqual(request2.new_email, 'alice2@example.com')
        self.failIf(request2.has_expired())
        request2.date -= datetime.timedelta(days=self.timeout_days + 1)
        request2.save()
        new = 'alice2@example.com'
        request2 = EmailChange.objects.filter(new_email=new).get()
        self.failUnless(request2.has_expired())
        management.call_command('cleanupemailchangerequests')
        self.assertEqual(EmailChange.objects.count(), 1)
        self.assertEqual(EmailChange.objects.filter(new_email=new).count(), 0)
        new = 'bob2@example.com'
        self.assertEqual(EmailChange.objects.filter(new_email=new).count(), 1)
        request1.delete()
        request2.delete()

    def test_email_address_change_managers(self):
        """
        Testing the model managers.

        """
        request1 = EmailChange.objects.create(new_email='bob2@example.com',
                                              user=self.bob)
        self.assertEqual(request1.user.id, self.bob.id)
        self.assertEqual(request1.new_email, 'bob2@example.com')
        self.failIf(request1.has_expired())
        request2 = EmailChange.objects.create(new_email='alice2@example.com',
                                              user=self.alice)
        self.assertEqual(request2.user.id, self.alice.id)
        self.assertEqual(request2.new_email, 'alice2@example.com')
        self.failIf(request2.has_expired())
        request2.date -= datetime.timedelta(days=self.timeout_days + 1)
        request2.save()
        new_email = 'alice2@example.com'
        request2 = EmailChange.objects.filter(new_email=new_email).get()
        self.failUnless(request2.has_expired())
        self.assertEqual(EmailChange.expired_objects.count(), 1)
        self.assertEqual(EmailChange.pending_objects.count(), 1)
        request1.delete()
        request2.delete()



from django.test import TestCase

from change_email.conf import settings


class BaseTest(TestCase):

    def setUp(self):
        """
        Isolate all application specific settings.
        """
        output = super(BaseTest, self).setUp()
        settings.isolated = True
        return output

    def tearDown(self):
        """
        Restore settings to their original state.
        """
        settings.isolated = False
        settings.revert()
        return super(BaseTest, self).tearDown()



from django.dispatch import Signal


# A user has completed a change of email address.
email_change_confirmed = Signal(providing_args=["request"])

# A user has requested a change of email address.
email_change_created = Signal(providing_args=["request"])

# A user has deleted a change of email address.
email_change_deleted = Signal(providing_args=["request"])



from datetime import timedelta

from django.contrib.sites.models import RequestSite
from django.contrib.sites.models import Site
from django.core.mail import EmailMultiAlternatives
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from django.core.signing import Signer
from django.core.signing import BadSignature

from change_email.conf import settings
from change_email.managers import ExpiredEmailChangeManager
from change_email.managers import PendingEmailChangeManager


class EmailChange(models.Model):
    """
A model to temporarily store an email adress change request.
"""
    new_email = models.EmailField(help_text=_('The new email address that'
                                              ' still needs to be confirmed.'),
                                  verbose_name=_('new email address'),)
    date = models.DateTimeField(auto_now_add=True,
                                help_text=_('The date and time the email '
                                            'address change was requested.'),
                                verbose_name=_('date'),)
    user = models.OneToOneField(settings.AUTH_USER_MODEL,
                                help_text=_('The user that has requested the'
                                            ' email address change.'),
                                verbose_name=_('user'),)
    site = models.ForeignKey(Site, blank=True, null=True)

    objects = models.Manager()
    expired_objects = ExpiredEmailChangeManager()
    pending_objects = PendingEmailChangeManager()

    class Meta:
        verbose_name = _('email address change request')
        verbose_name_plural = _('email address change requests')
        get_latest_by = "date"

    def __unicode__(self):
        return "%s" % self.user

    def get_absolute_url(self):
        return reverse('change_email_detail', kwargs={'pk': self.pk})

    def has_expired(self, seconds=None):
        """
Checks whether this request has already expired.

:kwarg int seconds: The number of seconds to calculate a
    :py:class:`datetime.timedelta` object.
    Defaults to :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_TIMEOUT`.
:returns: ``True`` if the request has already expired,
    ``False`` otherwise.
:rtype: bool
"""
        if not seconds:
            seconds = settings.EMAIL_CHANGE_TIMEOUT
        delta = timedelta(seconds=seconds)
        expiration_date = timezone.now() - delta
        return expiration_date >= self.date

    def check_signature(self, signature):
        """
Checks if

- the signature has not expired by calling :func:`has_expired`.
- the signature has not been tampered with by
  calling :func:`verify_signature`.

:arg str signature: The signature to check, as generated
    by :func:`make_signature`.
:returns: ``True`` if the check was successfully completed,
    ``False`` otherwise.
:rtype: bool
"""
        if not self.has_expired():
            return self.verify_signature(signature)
        return False

    def get_expiration_date(self, seconds=None):
        """
Returns the expiration date of an :model:`EmailChange` object by adding
a given amount of seconds to it.

:kwarg int seconds: The number of seconds to calculate a
    :py:class:`datetime.timedelta` object.
    Defaults to :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_TIMEOUT`.
:returns:  A :py:class:`datetime` object representing the expiration
    date.
:rtype: :py:obj:`.datetime`
"""
        if not seconds:
            seconds = settings.EMAIL_CHANGE_TIMEOUT
        delta = timedelta(seconds=seconds)
        return self.date + delta

    def make_signature(self):
        """
Generates a signature to use in one-time secret URL's
to confirm the email address change request.

:returns: A signature.
:rtype: str
"""
        signer = Signer()
        value = signer.sign(self.new_email)
        email, signature = value.split(':',  1)
        return signature

    def send_confirmation_mail(self, request):
        """
An instance method to send a confirmation mail to the new
email address.

The generation of a confirmation email will use three templates that
can be set in each project's settings:

* :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_HTML_EMAIL_TEMPLATE`.
* :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_SUBJECT_EMAIL_TEMPLATE`
* :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_TXT_EMAIL_TEMPLATE`

These templates will receive the following context variables:

``date``
    The date when the email address change was requested.

``timeout_date``
    The date whe the request will expire.

``current_site``
    An object representing the current site on which the user
    is logged in.  Depending on whether ``django.contrib.sites``
    is installed, this may be an instance of either
    ``django.contrib.sites.models.Site`` (if the sites
    application is installed) or
    ``django.contrib.sites.models.RequestSite`` (if
    not). Consult the documentation for the Django sites
    framework for details regarding these objects' interfaces.

``new_email``
    The new email address.

``protocol``
    The protocol used to generate the confirmation URL, either HTTP or HTTPS.
    To use HTTPS set :py:attr:`~password_policies.conf.Settings.EMAIL_CHANGE_USE_HTTPS`
    to True.

``signature``
    The confirmation signature for the new email address.

``user``
    The user that has requested the email address change.

:arg obj request: The request object.
"""
        if Site._meta.installed:
            current_site = Site.objects.get_current()
        else:
            current_site = RequestSite(request)
        subject = settings.EMAIL_CHANGE_SUBJECT_EMAIL_TEMPLATE
        body_htm = settings.EMAIL_CHANGE_HTML_EMAIL_TEMPLATE
        body_txt = settings.EMAIL_CHANGE_TXT_EMAIL_TEMPLATE
        context = {'current_site': current_site,
                    'date': self.date,
                    'timeout_date': self.get_expiration_date(),
                    'new_email': self.new_email,
                    'protocol': settings.EMAIL_CHANGE_USE_HTTPS and 'https' or 'http',
                    'signature': self.make_signature(),
                    'user': self.user}
        subject = render_to_string(subject, context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = render_to_string(body_txt, context)
        if settings.EMAIL_CHANGE_HTML_EMAIL:
            html_message = render_to_string(body_htm, context)
            msg = EmailMultiAlternatives(subject, text_message,
                                         settings.EMAIL_CHANGE_FROM_EMAIL,
                                         [self.new_email])
            msg.attach_alternative(html_message, "text/html")
            msg.send()
        else:
            send_mail(subject, text_message,
                      settings.EMAIL_CHANGE_FROM_EMAIL,
                      [self.new_email])

    def verify_signature(self, signature):
        """
Checks if the signature has been tampered with.

:arg str signature: The signature to check, as generated by
    :func:`make_signature`.
:returns: ``True`` if the signature has not been tampered with,
    ``False`` otherwise.
    :rtype: bool
    """
        signer = Signer()
        value = "%s:%s" % (self.new_email, signature)
        try:
            signer.unsign(value)
        except BadSignature:
            return False
        return True



import os

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
        'TEST_NAME': ':memory:',
        "USER": '',
        "PASSWORD": '',
        "PORT": '',
    },
}

DEBUG = False

LANGUAGES = (
    ('en', 'English'),
)

LANGUAGE_CODE = 'en'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'django.contrib.messages',
    'change_email',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'filters': {
    },
    'handlers': {
        'null': {
            'level':'DEBUG',
            'class':'django.utils.log.NullHandler',
        },
        'console':{
            'level':'DEBUG',
            'class':'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        'django': {
            'handlers':['null'],
            'propagate': True,
            'level':'INFO',
        },
        'django.request': {
            'handlers': ['console'],
            'level': 'ERROR',
            'propagate': False,
        },
        'change_email.views': {
            'handlers': ['null'],
            'level': 'INFO',
            'propagate': False,
        }
    }
}

ROOT_URLCONF = 'change_email.tests.urls'

SECRET_KEY = 'secret'

SITE_ID = 1

TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), 'templates'),
)

TEST_RUNNER = 'django.test.simple.DjangoTestSuiteRunner'

USE_TZ=False



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from change_email.models import EmailChange


def resend_confirmation(modeladmin, request, queryset):
    """
An admin action function to resend confirmation mails to
pending change request addresses.
"""
    for object in queryset:
        object.send_confirmation_mail()
resend_confirmation.short_description = _("Resend confirmation email"
                                          " to selected addresses")


class EmailChangeAdmin(admin.ModelAdmin):
    """
A ModelAdmin class to represent an :model:`EmailChange` model in the
admin interface.
"""
    actions = [resend_confirmation]
    date_hierarchy = 'date'
    list_display = ('id', 'user', 'new_email', 'date')
    list_display_links = ('id', 'user',)
    list_editable = ('new_email',)
    search_fields = ('new_email', 'id', 'user__id', 'user__username',
                     'user__first_name', 'user__last_name', 'user__email',)

    def get_readonly_fields(self, request, obj=None):
        """
Sets the ``user`` and the ``date`` field to read only if an instance
already exists.
"""
        if obj:
            return ['user']
        else:
            return []


admin.site.register(EmailChange, EmailChangeAdmin)



import os

from django.contrib.auth.models import User
from django.test import TestCase

from change_email import forms
from change_email.models import EmailChange
from change_email.tests.lib import BaseTest


class EmailChangeFormTestCase(BaseTest):

    fixtures = ['django_change_email_test_forms_fixtures.json']

    def test_change_email_form(self):
        """
        Test that ``EmailChangeForm`` enforces an email address that is
        not already taken by an existing user or a pending email address
        change request.

        """
        msg = u"This email address is already in use."
        msg = msg + u" Please supply a different email address."
        alice = User.objects.get(username='alice')
        bob = User.objects.get(username='bob')
        pending_request = EmailChange.objects.get(user=bob)
        invalid_data_dicts = [
            # Invalid email.
            {'data': {'new_email': 'alice'},
            'error': ('new_email', [u"Enter a valid email address."])},
            # Existing user.
            {'data': {'new_email': 'alice@example.com'},
            'error': ('new_email', [msg])},
            # Existing email address change request.
            {'data': {'new_email': 'bob2@example.com'},
            'error': ('new_email', [msg])},
            ]

        for invalid_dict in invalid_data_dicts:
            form = forms.EmailChangeForm(data=invalid_dict['data'])
            self.failIf(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])
        form = forms.EmailChangeForm(data={'new_email': 'alice2@example.com'})
        self.failUnless(form.is_valid())



from django.core.management.base import NoArgsCommand

from change_email.models import EmailChange


class Command(NoArgsCommand):
    """
The ``cleanupemailchangerequests`` command deletes expired email
address change requests from the database.

Usage::

    $ python manage.py cleanupemailchangerequests
"""
    help = "Delete expired email change requests from the database"

    def handle_noargs(self, **options):
        EmailChange.expired_objects.all().delete()



from django.conf import settings as django_settings

from easysettings import AppSettings


class Settings(AppSettings):
    """
Default settings for django-change-email.
"""
    #: Determines the URL to redirect to after an email change request has been
    #: deleted.
    EMAIL_CHANGE_DELETE_SUCCESS_REDIRECT_URL = '/account/email/change/'
    #: Determines the e-mail address field on the (custom) user model.
    EMAIL_CHANGE_FIELD = 'email'
    #: Determines the e-mail address used to send confirmation mails.
    EMAIL_CHANGE_FROM_EMAIL = django_settings.DEFAULT_FROM_EMAIL
    #: Determines wether to send HTML emails.
    EMAIL_CHANGE_HTML_EMAIL = False
    #: Determines the template used to render the HTML text of the
    #: confirmation email.
    EMAIL_CHANGE_HTML_EMAIL_TEMPLATE = 'change_email/mail/body.html'
    #: Determines the template used to render the subject of the
    #: confirmation email.
    EMAIL_CHANGE_SUBJECT_EMAIL_TEMPLATE = 'change_email/mail/subject.txt'
    #: Determines the expiration time of an e-mail address change requests.
    #: Defaults to 7 days.
    EMAIL_CHANGE_TIMEOUT = 60*60*24*7
    #: Determines the template used to render the plain text body of the
    #: confirmation email.
    EMAIL_CHANGE_TXT_EMAIL_TEMPLATE = 'change_email/mail/body.txt'
    #: Determines wether to use HTTPS when generating the confirmation link.
    EMAIL_CHANGE_USE_HTTPS = False
    #: Determines wether to check if the email address is used on a particular
    #: site. Set to True to make email addresses unique on different sites.
    EMAIL_CHANGE_VALIDATE_SITE = False


settings = Settings()



from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse

from change_email.forms import EmailChangeForm
from change_email.models import EmailChange
from change_email.tests.lib import BaseTest


class EmailChangeViewsTestCase(BaseTest):

    fixtures = ['django_change_email_test_views_fixtures.json']

    def setUp(self):
        output = super(EmailChangeViewsTestCase, self).setUp()
        self.alice = User.objects.get(username='alice')
        self.bob = User.objects.get(username='bob')
        self.client.login(username='bob', password='Oor0ohf4bi-')
        return output

    def test_email_address_change_creation(self):
        """
        A ``GET`` to the ``change_email_create`` view uses the appropriate
        template and populates the email change form into the context.

        """
        response = self.client.get(reverse('change_email_create'))
        self.assertEqual(response.status_code, 200)
        self.failUnless(isinstance(response.context['form'],
                                   EmailChangeForm))
        self.assertTemplateUsed(response,
                                'change_email/emailchange_form.html')

    def test_email_address_change_creation_success(self):
        """
        A ``POST`` to the ``change_email_create`` view with valid data properly
        creates a new email address change request and issues a redirect.

        """
        response = self.client.post(reverse('change_email_create'),
                                    data={'new_email': 'bob2@example.com'})
        self.assertEqual(EmailChange.objects.count(), 1)
        object = EmailChange.objects.get()
        self.assertEqual(len(mail.outbox), 1)
        self.assertRedirects(response,
                             'http://testserver%s' % reverse('change_email_detail',
                                                             args=[object.id]))
        object.delete()
        mail.outbox = []

    def test_email_address_change_creation_failure(self):
        """
        A ``POST`` to the ``change_email_create`` view with invalid data properly
        fails and issues the according error.

        """
        msg = u"This email address is already in use."
        msg = msg + u" Please supply a different email address."
        response = self.client.post(reverse('change_email_create'),
                                    data={'new_email': 'bob@example.com'})
        self.assertEqual(response.status_code, 200)
        self.failIf(response.context['form'].is_valid())
        self.assertFormError(response, 'form', field='new_email',
                             errors=msg)
        self.assertEqual(len(mail.outbox), 0)
        response = self.client.post(reverse('change_email_create'),
                                    data={'new_email': 'alice@example.com'})
        self.assertEqual(response.status_code, 200)
        self.failIf(response.context['form'].is_valid())
        self.assertFormError(response, 'form', field='new_email',
                             errors=msg)
        self.assertEqual(len(mail.outbox), 0)

    def test_email_address_change_creation_already_existing(self):
        """
        A ``POST`` to the ``change_email_create`` view when a pending request
        already exists issues a redirect to the ``change_email_detail`` view
        of the existing request.

        """
        request = EmailChange.objects.create(new_email='bob2@example.com',
                                                          user=self.bob)
        response = self.client.post(reverse('change_email_create'),
                                    data={'new_email': 'bob2@example.com'})
        self.assertRedirects(response,
                             'http://testserver%s' % reverse('change_email_detail',
                                                             args=[request.id]))
        request.delete()

    def test_email_address_change_confirmation_success(self):
        """
        A ``GET`` to the ``change_email_confirm`` view with the valid signature works.
        """
        request = EmailChange.objects.create(new_email='bob2@example.com',
                                                          user=self.bob)
        signature = request.make_signature()
        response = self.client.get(reverse('change_email_confirm', args=[signature,]))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.context['confirmed'])
        bob = User.objects.filter(username='bob').get()
        self.assertEqual(bob.email, 'bob2@example.com')
        self.assertEqual(EmailChange.objects.filter(new_email='bob2@example.com').count(), 0)

    def test_email_address_change_confirmation_failure(self):
        """
        A ``GET`` to the ``change_email_confirm`` view with an invalid signature does not work.
        """
        request = EmailChange.objects.create(new_email='bob2@example.com',
                                                          user=self.bob)
        signature = 'foo'
        response = self.client.get(reverse('change_email_confirm', args=[signature,]))
        self.assertEqual(response.status_code, 200)
        self.assertFalse(response.context['confirmed'])
        self.assertEqual(EmailChange.objects.filter(new_email='bob2@example.com').count(), 1)
        request.delete()

    def test_email_address_change_confirmation_non_existing(self):
        """
        A ``GET`` to the ``change_email_confirm`` view when no pending request
        exist issues a redirect to the ``change_email_create`` view.

        """
        signature = 'foo'
        response = self.client.get(reverse('change_email_confirm', args=[signature,]))
        self.assertEqual(response.status_code, 200)
        self.assertFalse(response.context['confirmed'])

    def test_email_address_change_deletion(self):
        """
        A ``GET`` to the ``change_email_delete`` view works.

        """
        request = EmailChange.objects.create(new_email='bob2@example.com',
                                                          user=self.bob)
        response = self.client.get(reverse('change_email_delete',
                                           args=[request.id,]))
        self.assertEqual(response.status_code, 200)
        request.delete()

    def test_email_address_change_deletion_non_existing(self):
        """
        A ``GET`` to the ``change_email_delete`` view when no pending request
        exist issues a redirect to the ``change_email_create`` view.

        """
        response = self.client.get(reverse('change_email_delete', args=[1,]))
        self.assertRedirects(response,
                             'http://testserver%s' % reverse('change_email_create'))

    def test_email_address_change_deletion_success(self):
        """
        A ``POST`` to the ``change_email_delete`` view with valid data works.

        """
        request = EmailChange.objects.create(new_email='bob2@example.com', user=self.bob)
        response = self.client.post(reverse('change_email_delete',
                                            args=[request.id,]))
        self.assertEqual(EmailChange.objects.filter(new_email='bob2@example.com').count(), 0)
        request.delete()

    def test_email_address_change_detail(self):
        """
        A ``GET`` to the ``change_email_detail`` view with valid data works.

        """
        request = EmailChange.objects.create(new_email='bob2@example.com', user=self.bob)
        response = self.client.get(reverse('change_email_detail', args=[request.id,]))
        self.assertEqual(response.status_code, 200)
        request.delete()

    def test_email_address_change_detail_non_existing(self):
        """
        A ``GET`` to the ``change_email_detail`` view when no pending request
        exist issues a redirect to the ``change_email_create`` view.


        """
        response = self.client.get(reverse('change_email_detail', args=[1000]))
        self.assertRedirects(response,
                             'http://testserver%s' % reverse('change_email_create'))



from setuptools import setup, find_packages

setup(
    name='django-change-email',
    version=__import__('change_email').__version__,
    description='A Django application to enable users to change their e-mail address.',
    long_description="""django-change-email enables registered users to change
their e-mail address.""",
    author='Tarak Blah',
    author_email='halbkarat@gmail.com',
    url='https://github.com/tarak/django-change-email',
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=['django>=1.5,<=1.6',
                      'django-easysettings',
    ],
    test_suite='tests.main',
)



from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from change_email.conf import settings
from change_email.models import EmailChange


class EmailNotUsedValidator(object):
    """
A validator to check if a given email address is already taken.
"""
    code = "email_in_use"
    msg = _("This email address is already in use."
            " Please supply a different email address.")

    def __call__(self, value):
        UserModel = get_user_model()
        key = '%s__iexact' % settings.EMAIL_CHANGE_FIELD
        kwargs = {key: value}
        if settings.EMAIL_CHANGE_VALIDATE_SITE:
            site = Site.objects.get_current()
            kwargs['site'] = site
        if UserModel.objects.filter(**kwargs).count():
            raise ValidationError(self.msg, code=self.code)
            return
        del kwargs[key]
        kwargs['new_email__iexact'] = value
        if EmailChange.objects.filter(**kwargs).count():
            raise ValidationError(self.msg, code=self.code)

validate_email_not_used = EmailNotUsedValidator()
